;;; matrix:back-substitute
;;;
;;; Performs back substitution on the (assumed
;;; upper-triangular) matrix using the right-hand side
;;; vectors (in matrix form).

(define (matrix:back-substitute a rhs)
  (cond
   ((not (matrix? a))
    (matrix:error 'matrix:back-substitute "~s is not a matrix" a))
   ((not (matrix? rhs))
    (matrix:error 'matrix:back-substitute "~s is not a matrix" rhs))
   (else
    (let* ((rows (matrix:rows rhs))
	   (cols (matrix:cols rhs))
	   (m (matrix:create rows cols (lambda (i j) 0)))
	   (term
	    (lambda (i vec-n)
	      (if (= i (1- rows))
		  (/ (matrix:ref rhs (1- rows) vec-n) (matrix:ref a (1- rows) (1- rows)))
		  (/ (- (matrix:ref rhs i vec-n)
			(let loop ((sum 0) (j (1+ i)))
			  (cond
			   ((= j (matrix:cols a)) sum)
			   (else (loop (+ sum (* (matrix:ref a i j)
						 (matrix:ref m j vec-n)))
				       (1+ j))))))
		     (matrix:ref a i i))))))
      
      (do ((i (1- rows) (1- i))) ((= i -1) m)
	(do ((j 0 (1+ j))) ((= j cols))
	  (matrix:set! m i j (term i j))))))))

