(require 'determinant)

(define a 0)

(define b 1)
(define n 10)
(define (a1 t) 1)
(define (a2 t) 2)
(define a3 -10)
(define y0 2)
(define yn -5)

(define (u0 x) (+ (* (exp (* -1 x)) 7) 4))
(define nn (+ n 1))
(define h (/ (- b a) n))
(define x (matrix:create nn 1 (lambda (i j) (+ (* i h) a))))

(define (L_1 i j) (if (and (= i j) (> i 0) (< i n)) -2 0))
(define (L_2 i j) (if (and (= (- i 1) j) (> i 0) (< i n)) 1 0))
(define (L_3 i j) (if (and (= (- j 1) i) (> i 0) (< i n)) 1 0))

(define L (matrix:create nn nn L_1))
(set! L (matrix:sum L (matrix:create nn nn L_2)))
(set! L (matrix:sum L (matrix:create nn nn L_3)))
(set! L (matrix:product L (/ 1 (* h h))))
(set! L (matrix:create nn nn (lambda (i j) (* (a1 (matrix:ref x i 0)) (matrix:ref L i j)))))

(define (M_1 i j) (if (and (= (- i 1) j) (> i 0) (< i n)) -1 0))
(define (M_2 i j) (if (and (= (- j 1) i) (> i 0) (< i n)) 1 0))

(define M (matrix:create nn nn M_1))
(set! M (matrix:sum M (matrix:create nn nn M_2)))

(set! M (matrix:product M (/ 1 (* 2 h))))
(set! M (matrix:create nn nn (lambda (i j) (* (a2 (matrix:ref x i 0)) (matrix:ref M i j)))))

(define (HH_1 i j) (if (and (= i j) (> i 0) (< i n)) a3 0))
(define HH (matrix:create nn nn HH_1))

(set! L (matrix:sum L M))
(set! L (matrix:sum L HH))

(matrix:set! L 0 0 1)
(matrix:set! L n n 1)

;; build right-hand-side
(define rhs (matrix:create nn 1 (lambda (i j) (u0 (matrix:ref x i j)))))
(matrix:set! rhs 0 0 y0)
(matrix:set! rhs n 0 yn)

;; solver
(define start-time (current-time))

(define PLU (matrix:plu-factor L))
(define y (matrix:forward-substitute (cadr PLU) (matrix:permute-rows rhs (car PLU))))
(set! y (matrix:back-substitute (cddr PLU) y))

(define proc-time (- (current-time) start-time))


;; statistics
(let loop ((i 0))
	(cond ((< i nn)
		(display (matrix:ref x i 0)) (display "\t\t\t\t") (display (matrix:ref y i 0))
		(display "\n")
		(loop (+ i 1)))))
(display "runtime: ")
(display proc-time)
(display " second(s)\n")
