
(require 'determinant)

(define alpha 0.3)
(define beta 2.0)
(define gamma 0.8)
(define delta 1.0)
(define a 0)
(define b 200)
(define h 2)
(define T 40)
(define delt (/ 1 96))
(define (u0 x) (/ (exp (* -1 (expt (- x 100) 2))) 5))
(define v0 (/ 2 5))

;; Calculate some constants

(define mu (/ delt (expt h 2)))
(define jh (inexact->exact (/ (- b a) h)))
(define nn (+ jh 0))
(define N (round (/ T delt)))

;; Initialization

(define u (matrix:create nn 1 0))
(define v (matrix:create nn 1 0))
(define F (matrix:create nn 1 0))
(define G (matrix:create nn 1 0))
(define y1 (matrix:create nn 1 0))
(define y2 (matrix:create nn 1 0))
(define z1 (matrix:create nn 1 0))
(define z2 (matrix:create nn 1 0))
(define B1 (matrix:create nn nn 0))
(define B2 (matrix:create nn nn 0))
(define L (matrix:create nn nn 0))
(define Lower1 (matrix:create nn nn 0))
(define Lower2 (matrix:create nn nn 0))
(define U1 (matrix:create nn nn 0))
(define U2 (matrix:create nn nn 0))

;; Assign initial data

(define x (matrix:create nn 1 (lambda (i j) (+ (* i h) a))))
(define u (matrix:create nn 1 (lambda (i j) (u0 (matrix:ref x i j)))))
(define v (matrix:create nn 1 v0))

;; Construct the matrix L(without the 1/h^2 factor

(define (L_1 i j) (if (and (= i 0) (= j 1)) -2 0))
(define (L_2 i j) (if (and (= i (1- nn)) (= j (- nn 2))) -2 0))
(define (L_3 i j) 
	(if (and (= i (- j 1)) (>= i 1) (<= i (- nn 2)) (>= j 2) (<= j (1- nn))) -1 0))
(define (L_4 i j)
	(if (and (= i (+ j 1)) (>= i 1) (<= i (- nn 2)) (>= j 0) (<= j (- nn 3))) -1 0))
(define (L_5 i j)
	(if (and (= i j) (>= i 0) (<= i (1- nn)) (>= j 0) (<= j (1- nn))) 2 0))

(define L (matrix:sum L (matrix:create nn nn L_1)))
(define L (matrix:sum L (matrix:create nn nn L_2)))
(define L (matrix:sum L (matrix:create nn nn L_3)))
(define L (matrix:sum L (matrix:create nn nn L_4)))
(define L (matrix:sum L (matrix:create nn nn L_5)))

;; Construct matrices B1 and B2

(define (B_fn i j)
	(if (and (= i j) (>= i 0) (<= i (1- nn)) (>= j 0) (<= j (1- nn))) 1 0))
(define B1 (matrix:sum (matrix:create nn nn B_fn) (matrix:product L mu)))
(define B2 (matrix:sum (matrix:create nn nn B_fn) (matrix:product L (* mu delta))))

(define PLU1 (matrix:plu-factor B1))
(define PLU2 (matrix:plu-factor B2))

;; Time-stepping procedure

(define hhat (matrix:create nn 1 0))
(define F (matrix:create nn 1 0))
(define G (matrix:create nn 1 0))
(define y1 (matrix:create nn 1 0))
(define y2 (matrix:create nn 1 0))
(define z1 (matrix:create nn 1 0))
(define z2 (matrix:create nn 1 0))


(let time-step ((t 1))
	(cond ((< t N)
		(matrix:mset! hhat 
			(lambda (i j) (/ (matrix:ref u i j) (+ alpha (magnitude (matrix:ref u i j))))))
		(matrix:mset! F
			(lambda (i j) (- (matrix:ref u i j) (* (matrix:ref u i j) (magnitude (matrix:ref u i j))) 
				(* (matrix:ref hhat i j) (matrix:ref v i j)))))
		(matrix:mset! G
			(lambda (i j) (- (* beta (matrix:ref v i j) (matrix:ref hhat i j))
				(* gamma (matrix:ref hhat i j)))))
		(matrix:mset! y1
			(lambda (i j) (+ (matrix:ref u i j) (* delt (matrix:ref F i j)))))
		(matrix:mset! y2
			(lambda (i j) (+ (matrix:ref v i j) (* delt (matrix:ref G i j)))))

		;; forward substitution
		(set! z1 (matrix:forward-substitute (cadr PLU1) (matrix:permute-rows y1 (car PLU1))))

		;; backward substitute
		(set! u (matrix:back-substitute (cddr PLU1) z1))

		(set! z2 (matrix:forward-substitute (cadr PLU2) (matrix:permute-rows y2 (car PLU2))))

		(set! v (matrix:back-substitute (cddr PLU2) z2))
		(time-step (+ t 1)))))
