;;; matrix:forward-substitute
;;;
;;; Performs forward substitution on the (assumed
;;; upper-triangular) matrix and the righthand side vectors
;;; (in matrix form).  The method used is simple; just use
;;; backward substitution after mirroring the matrices.  The
;;; matrix itself has to be sort of flipped along the diagonal;
;;; the matrix of right-hand side vectors just has to be
;;; flipped upside-down.  Of course, once the set of solution
;;; vectors is found, it must be flipped back over.

(define (matrix:forward-substitute a rhs)
  (cond
   ((not (matrix? a))
    (matrix:error 'matrix:forward-substitute "~s is not a matrix" a))
   ((not (matrix? rhs))
    (matrix:error 'matrix:forward-substitute "~s is not a matrix" rhs))
   ((not (= (matrix:rows a) (matrix:rows rhs)))
    (matrix:error 'matrix:forward-substitute
	   "matrices have differing numbers of rows"))
   (else
    (let* ((rows (matrix:rows a))
	   (cols (matrix:cols a))
	   
	   ;; mirror-matrix 
	   (mm (lambda (a)
		 (matrix:create rows cols
				(lambda (i j)
				  (matrix:ref
				   a (- rows i 1) (- cols j 1))))))
	   ;; mirror-vector-matrix
	   (mvm (lambda (a)
		  (matrix:create rows (matrix:cols rhs)
				 (lambda (i j)
				   (matrix:ref a (- rows i 1) j))))))
      
      (mvm (matrix:back-substitute (mm a) (mvm rhs)))))))

